﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Infragistics.Windows.Themes;
using Infragistics.Controls.Editors;

namespace WPFLiveStylableApp
{
    /// <summary>
    /// Interaction logic for ThemingManager.xaml
    /// </summary>
    public partial class ThemingManager : Window
    {
        private bool isLoaded = false;
        private Color selectedWashColor = Colors.Transparent;
        private List<ResourceWasher> washerList = new List<ResourceWasher>();
        private List<ResourceDictionary> themeList = new List<ResourceDictionary>();

        public ThemingManager()
        {
            InitializeComponent();
            colorBlock.Background = new SolidColorBrush(selectedWashColor);

            // Register the WashBaseLight and WashBaseDark themes for all controls
            ThemeManager.Register("WashBaseLight", "Ribbon", RibbonWashBaseLight.Instance);
            ThemeManager.Register("WashBaseDark", "Ribbon", RibbonWashBaseDark.Instance);

            ThemeManager.Register("WashBaseLight", "DockManager", DockManagerWashBaseLight.Instance);
            ThemeManager.Register("WashBaseDark", "DockManager", DockManagerWashBaseDark.Instance);

            ThemeManager.Register("WashBaseLight", "DataPresenter", DataPresenterWashBaseLight.Instance);
            ThemeManager.Register("WashBaseDark", "DataPresenter", DataPresenterWashBaseDark.Instance);

            ThemeManager.Register("WashBaseLight", "Editors", EditorsWashBaseLight.Instance);
            ThemeManager.Register("WashBaseDark", "Editors", EditorsWashBaseDark.Instance);

            ThemeManager.Register("WashBaseLight", "Primitives", PrimitivesWashBaseLight.Instance);
            ThemeManager.Register("WashBaseDark", "Primitives", PrimitivesWashBaseDark.Instance);

            ThemeManager.Register("WashBaseLight", "Tiles", TilesWashBaseLight.Instance);
            ThemeManager.Register("WashBaseDark", "Tiles", TilesWashBaseDark.Instance);

            ThemeManager.Register("WashBaseLight", "OutlookBar", OutlookBarWashBaseLight.Instance);
            ThemeManager.Register("WashBaseDark", "OutlookBar", OutlookBarWashBaseDark.Instance);

            Loaded += new RoutedEventHandler(ThemingManager_Loaded);
        }

        void ThemingManager_Loaded(object sender, RoutedEventArgs e)
        {
            isLoaded = true;
            colorPicker.SelectedColor = selectedWashColor;
        }


        private void WashResources()
        {
            if (!isLoaded)
                return;

            //remove the current resource washers from application resources
            foreach (ResourceWasher washer in washerList)
                Application.Current.Resources.MergedDictionaries.Remove(washer);
            washerList.Clear();

            //remove current theme resources from application resources
            foreach (ResourceDictionary themeResource in themeList)
                Application.Current.Resources.MergedDictionaries.Remove(themeResource);
            themeList.Clear();

            string selectedWashBase = cbxWashBase.SelectedItem as string;//"RibbonWashBaseLight";
            if (String.IsNullOrEmpty(selectedWashBase))
                return;

            switch (selectedWashBase)
            {
                case "WashBaseLight":
                    // inject theme+washer into the application merged dictionaries for every type of control themed
                    AddResourcesToMergedDictionary("WashBaseLight", "Ribbon", new RibbonWashBaseLightBrushes());
                    AddResourcesToMergedDictionary("WashBaseLight", "DataPresenter", new DataPresenterWashBaseLightBrushes());
                    AddResourcesToMergedDictionary("WashBaseLight", "DockManager", new DockManagerWashBaseLightBrushes());
                    AddResourcesToMergedDictionary("WashBaseLight", "Editors", new EditorsWashBaseLightBrushes());
                    AddResourcesToMergedDictionary("WashBaseLight", "Tiles", new TilesWashBaseLightBrushes());
                    AddResourcesToMergedDictionary("WashBaseLight", "Primitives", new PrimitivesWashBaseLight());
                    AddResourcesToMergedDictionary("WashBaseLight", "OutlookBar", new OutlookBarWashBaseLightBrushes());
                    break;

                case "WashBaseDark":
                    // inject theme+washer into the application merged dictionaries for every type of control themed
                    AddResourcesToMergedDictionary("WashBaseDark", "Ribbon", new RibbonWashBaseDarkBrushes());
                    AddResourcesToMergedDictionary("WashBaseDark", "DataPresenter", new DataPresenterWashBaseDarkBrushes());
                    AddResourcesToMergedDictionary("WashBaseDark", "DockManager", new DockManagerWashBaseDarkBrushes());
                    AddResourcesToMergedDictionary("WashBaseDark", "Editors", new EditorsWashBaseDarkBrushes());
                    AddResourcesToMergedDictionary("WashBaseDark", "Tiles", new TilesWashBaseDarkBrushes());
                    AddResourcesToMergedDictionary("WashBaseDark", "Primitives", new PrimitivesWashBaseDark());
                    AddResourcesToMergedDictionary("WashBaseDark", "OutlookBar", new OutlookBarWashBaseDarkBrushes());
                    break;

                case "Office2k7Black":
                    // inject theme+washer into the application merged dictionaries for every type of control themed
                    AddResourcesToMergedDictionary("Office2k7Black", "Ribbon", new RibbonOffice2k7BlackBrushes());
                    AddResourcesToMergedDictionary("Office2k7Black", "DataPresenter", new DataPresenterOffice2k7BlackBrushes());
                    AddResourcesToMergedDictionary("Office2k7Black", "DockManager", new DockManagerOffice2k7BlackBrushes());
                    AddResourcesToMergedDictionary("Office2k7Black", "Editors", new EditorsOffice2k7BlackBrushes());
                    AddResourcesToMergedDictionary("Office2k7Black", "Tiles", new TilesOffice2k7BlackBrushes());
                    AddResourcesToMergedDictionary("Office2k7Black", "Primitives", new PrimitivesOffice2k7Black());
                    AddResourcesToMergedDictionary("Office2k7Black", "OutlookBar", new OutlookBarOffice2k7BlackBrushes());
                    break;
                case "Office2k7Silver":
                    // inject theme+washer into the application merged dictionaries for every type of control themed
                    AddResourcesToMergedDictionary("Office2k7Silver", "Ribbon", new RibbonOffice2k7SilverBrushes());
                    AddResourcesToMergedDictionary("Office2k7Silver", "DataPresenter", new DataPresenterOffice2k7SilverBrushes());
                    AddResourcesToMergedDictionary("Office2k7Silver", "DockManager", new DockManagerOffice2k7SilverBrushes());
                    AddResourcesToMergedDictionary("Office2k7Silver", "Editors", new EditorsOffice2k7SilverBrushes());
                    AddResourcesToMergedDictionary("Office2k7Silver", "Tiles", new TilesOffice2k7SilverBrushes());
                    AddResourcesToMergedDictionary("Office2k7Silver", "Primitives", new PrimitivesOffice2k7Silver());
                    AddResourcesToMergedDictionary("Office2k7Silver", "OutlookBar", new OutlookBarOffice2k7SilverBrushes());
                    break;

                case "Office2k7Blue":
                    // inject theme+washer into the application merged dictionaries for every type of control themed
                    AddResourcesToMergedDictionary("Office2k7Blue", "Ribbon", new RibbonOffice2k7BlueBrushes());
                    AddResourcesToMergedDictionary("Office2k7Blue", "DataPresenter", new DataPresenterOffice2k7BlueBrushes());
                    AddResourcesToMergedDictionary("Office2k7Blue", "DockManager", new DockManagerOffice2k7BlueBrushes());
                    AddResourcesToMergedDictionary("Office2k7Blue", "Editors", new EditorsOffice2k7BlueBrushes());
                    AddResourcesToMergedDictionary("Office2k7Blue", "Tiles", new TilesOffice2k7BlueBrushes());
                    AddResourcesToMergedDictionary("Office2k7Blue", "Primitives", new PrimitivesOffice2k7Blue());
                    AddResourcesToMergedDictionary("Office2k7Blue", "OutlookBar", new OutlookBarOffice2k7BlueBrushes());
                    break;
            }
        }
        
        private void AddResourcesToMergedDictionary(string resourceName, string groupName, ResourceSet brushes)
        {
            //setup the resource washer using the settings provided by the user
            ResourceWasher themeWasher = new ResourceWasher();
            themeWasher.BeginInit();
            themeWasher.WashMode = (WashMode)cbxWashMode.SelectedItem;
            themeWasher.WashColor = selectedWashColor;
            themeWasher.SourceDictionary = brushes;
            themeWasher.EndInit();

            //retrieve the theming resource for this resource and group name
            ResourceDictionary themeResource = ThemeManager.GetResourceSet(resourceName, groupName);
            if (themeResource != null)
            {
                Application.Current.Resources.MergedDictionaries.Add(themeResource);
                themeList.Add(themeResource);
            }
            else
                MessageBox.Show("Resource " + resourceName + " from group " + groupName + " was not found", "Infragistics Theming Manager", MessageBoxButton.OK, MessageBoxImage.Error);

            Application.Current.Resources.MergedDictionaries.Add(themeWasher);
            washerList.Add(themeWasher);
        }
        
        private void cbxWashBase_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            //update the appearance in response to the newly selected base theme
            WashResources();
        }

        private void cbxWashMode_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            //update the appearance in response to the newly selected wash mode
            WashResources();
        }

        private void OnWashGroupColorChanged(object sender, SelectedColorChangedEventArgs e)
        {
            //update the appearance in response to the newly selected wash color
            if (e.NewColor != null)
            {
                selectedWashColor = e.NewColor.Value;
                border.Background = new SolidColorBrush(selectedWashColor);
                WashResources();
            }
        }
    }
}
